@@ -8,9 +8,11 @@ use yearfrac::DayCountConvention;
 /// default format is "dd/mm/yyyy"
 ///
 /// default DayCount Convention is Act360 (2 in Excel)
-pub fn drc_scalinng(dc: Option<u8>, format: Option<&String>) -> Expr {
-    let dc_int = dc.unwrap_or(2);
-    let dc = DayCountConvention::from_int(dc_int).unwrap();
+pub fn drc_scalinng(dc: Option<&String>, format: Option<&String>) -> Expr {
+    let dc_int = dc.and_then(|x| x.parse::<u8>().ok()).unwrap_or(2);
+    let dc = DayCountConvention::from_int(dc_int).expect("Couldn't parse Day count convention");
+    //Not sure why compiler doesn't like this
+    //let format = format.map(|x| &**x);
     let format = format.map(|x| x.to_owned());
     apply_multiple(
         move |columns| {
