@@ -3,14 +3,14 @@
 //! TODO Weights column is a list. We can't read it as list from a csv. frame_from_path_or_str must concat_lst columns which have Sens in the name
 
 use crate::drc::drc_weights;
-use base_engine::helpers::diag_concat_lf;
-use base_engine::polars::prelude::{
+use once_cell::sync::OnceCell;
+use std::collections::BTreeMap;
+use ultibi::polars::prelude::{
     col, concat_lst, concat_str, df, lit, CsvReader, DataFrame, DataType, Expr, GetOutput,
     IntoLazy, IntoSeries, JoinType, LazyFrame, NamedFrom, PolarsError, PolarsResult, SerReader,
     Series, Utf8NameSpaceImpl,
 };
-use once_cell::sync::OnceCell;
-use std::collections::HashMap;
+use ultibi::prelude::helpers::diag_concat_lf;
 
 static FX_SPECIAL_DELTA_FULL_RW: OnceCell<LazyFrame> = OnceCell::new();
 static FX_SPECIAL_DELTA_PARTIAL_RW: OnceCell<LazyFrame> = OnceCell::new();
@@ -56,7 +56,7 @@ pub struct SensWeightsConfig {
 /// Then calls [weight_assign_logic] where weights assignment actually happens
 pub fn weights_assign(
     lf: LazyFrame,
-    build_params: &HashMap<String, String>,
+    build_params: &BTreeMap<String, String>,
 ) -> PolarsResult<LazyFrame> {
     // check columns. Some of the cast weights files must contain these:
     let check_columns0 = [
@@ -549,6 +549,8 @@ pub fn weight_assign_logic(lf: LazyFrame, weights: SensWeightsConfig) -> PolarsR
         lf1 = lf1.rename(["Weights"], ["SensWeights"])
     }
 
+    //dbg!(lf1.clone().filter(col("RiskClass").eq(lit("FX"))).select([col("BucketBCBS"), col("RiskFactor"), col("SensWeights")]).collect());
+
     let join_on = [col("RiskClass"), col("RiskCategory"), col("BucketBCBS")];
     lf1 = lf1.join(
         weights.rc_rcat_b_weights,
