@@ -1,13 +1,13 @@
 //#![allow(clippy::unnecessary_lazy_evaluations)]
 
-use base_engine::prelude::*;
+use ultibi::prelude::*;
 
-use base_engine::polars::export::num::Signed;
-use base_engine::polars::prelude::{
+use ndarray::{s, Array1, Array2, ArrayView1, Axis, Zip};
+use ultibi::polars::export::num::Signed;
+use ultibi::polars::prelude::{
     apply_multiple, AnyValue, ChunkAgg, ChunkCompare, ChunkSet, DataType, FillNullStrategy,
     Float64Type, GetOutput, IntoSeries, NamedFrom, NumOpsDispatch, PolarsError, Series, TakeRandom,
 };
-use ndarray::{s, Array1, Array2, ArrayView1, Axis, Zip};
 
 use rayon::{
     iter::{ParallelBridge, ParallelIterator},
@@ -116,8 +116,8 @@ pub fn rc_tenor_weighted_sens(
 }
 
 ///makes sence at RiskClass-Bucket view
-pub fn sens_weights(_: &OCP) -> Expr {
-    col("SensWeights")
+pub fn sens_weights(_: &CPM) -> PolarsResult<Expr> {
+    Ok(col("SensWeights"))
 }
 
 pub(crate) fn across_bucket_agg<I: IntoIterator<Item = f64>>(
