@@ -1,16 +1,19 @@
 use crate::prelude::*;
-use base_engine::{
-    polars::prelude::{apply_multiple, col, df, lit, max_exprs, when, DataType, GetOutput},
-    OCP,
+use ultibi::{
+    polars::prelude::{
+        apply_multiple, col, df, lit, max_exprs, when, DataType, GetOutput, PolarsError,
+    },
+    BaseMeasure, IntoLazy, CPM,
 };
+
 //use polars::lazy::dsl::{apply_multiple, col, lit, when};
 use rayon::prelude::IntoParallelIterator;
 
 use crate::risk_weights::REDUCED_IR_WEIGHT;
 use ndarray::{parallel::prelude::ParallelIterator, Array1, Array2};
 
-pub fn total_ir_delta_sens(_: &OCP) -> Expr {
-    rc_rcat_sens("Delta", "GIRR", total_delta_sens())
+pub fn total_ir_delta_sens(_: &CPM) -> PolarsResult<Expr> {
+    Ok(rc_rcat_sens("Delta", "GIRR", total_delta_sens()))
 }
 /// Helper functions
 fn girr_delta_sens_weighted_spot() -> Expr {
@@ -48,8 +51,8 @@ fn girr_delta_sens_weighted_30y() -> Expr {
 }
 
 /// Total GIRR Delta Seins
-pub(crate) fn girr_delta_sens_weighted(_: &OCP) -> Expr {
-    girr_delta_sens_weighted_spot().fill_null(0.)
+pub(crate) fn girr_delta_sens_weighted(_: &CPM) -> PolarsResult<Expr> {
+    Ok(girr_delta_sens_weighted_spot().fill_null(0.)
         + girr_delta_sens_weighted_025y().fill_null(0.)
         + girr_delta_sens_weighted_05y().fill_null(0.)
         + girr_delta_sens_weighted_1y().fill_null(0.)
@@ -59,94 +62,103 @@ pub(crate) fn girr_delta_sens_weighted(_: &OCP) -> Expr {
         + girr_delta_sens_weighted_10y().fill_null(0.)
         + girr_delta_sens_weighted_15y().fill_null(0.)
         + girr_delta_sens_weighted_20y().fill_null(0.)
-        + girr_delta_sens_weighted_30y().fill_null(0.)
+        + girr_delta_sens_weighted_30y().fill_null(0.))
 }
 
 /// Interm Result: GIRR Delta Sb <--> Sb Low == Sb Medium == Sb High
-pub(crate) fn girr_delta_sb(op: &OCP) -> Expr {
+pub(crate) fn girr_delta_sb(op: &CPM) -> PolarsResult<Expr> {
     girr_delta_charge_distributor(op, &LOW_CORR_SCENARIO, ReturnMetric::Sb)
 }
 
 ///calculate GIRR Delta Low Capital charge
-pub(crate) fn girr_delta_charge_low(op: &OCP) -> Expr {
+pub(crate) fn girr_delta_charge_low(op: &CPM) -> PolarsResult<Expr> {
     girr_delta_charge_distributor(op, &LOW_CORR_SCENARIO, ReturnMetric::CapitalCharge)
 }
 /// Interm Result: GIRR Delta Kb Low
-pub(crate) fn girr_delta_kb_low(op: &OCP) -> Expr {
+pub(crate) fn girr_delta_kb_low(op: &CPM) -> PolarsResult<Expr> {
     girr_delta_charge_distributor(op, &LOW_CORR_SCENARIO, ReturnMetric::Kb)
 }
 
 ///calculate GIRR Delta Medium Capital charge
-pub(crate) fn girr_delta_charge_medium(op: &OCP) -> Expr {
+pub(crate) fn girr_delta_charge_medium(op: &CPM) -> PolarsResult<Expr> {
     girr_delta_charge_distributor(op, &MEDIUM_CORR_SCENARIO, ReturnMetric::CapitalCharge)
 }
 /// Interm Result: GIRR Delta Kb Medium
-pub(crate) fn girr_delta_kb_medium(op: &OCP) -> Expr {
+pub(crate) fn girr_delta_kb_medium(op: &CPM) -> PolarsResult<Expr> {
     girr_delta_charge_distributor(op, &MEDIUM_CORR_SCENARIO, ReturnMetric::Kb)
 }
 
 ///calculate GIRR Delta High Capital charge
-pub(crate) fn girr_delta_charge_high(op: &OCP) -> Expr {
+pub(crate) fn girr_delta_charge_high(op: &CPM) -> PolarsResult<Expr> {
     girr_delta_charge_distributor(op, &HIGH_CORR_SCENARIO, ReturnMetric::CapitalCharge)
 }
 /// Interm Result: GIRR Delta Kb High
-pub(crate) fn girr_delta_kb_high(op: &OCP) -> Expr {
+pub(crate) fn girr_delta_kb_high(op: &CPM) -> PolarsResult<Expr> {
     girr_delta_charge_distributor(op, &HIGH_CORR_SCENARIO, ReturnMetric::Kb)
 }
 
 /// Helper funciton
 /// Extracts relevant fields from OptionalParams
 fn girr_delta_charge_distributor(
-    op: &OCP,
+    op: &CPM,
     scenario: &'static ScenarioConfig,
     rtrn: ReturnMetric,
-) -> Expr {
-    let juri: Jurisdiction = get_jurisdiction(op);
-    let _suffix = scenario.as_str();
-    let rep_ccy = op.get("reporting_ccy").map_or_else(
-        || match juri {
+) -> PolarsResult<Expr> {
+    let juri: Jurisdiction = get_jurisdiction(op)?;
+
+    let rep_ccy = if let Some(s) = op.get("reporting_ccy") {
+        if s.len() == 3 {
+            Ok(s.to_owned())
+        } else {
+            Err(PolarsError::ComputeError(
+                "reporting_ccy must be of length 3: CCY".into(),
+            ))
+        }
+    } else {
+        match juri {
             #[cfg(feature = "CRR2")]
-            Jurisdiction::CRR2 => "EUR".to_string(),
-            _ => "USD".to_string(),
-        },
-        |ccy| ccy.to_string(),
-    );
+            Jurisdiction::CRR2 => Ok("EUR".to_string()),
+            _ => Ok("USD".to_string()),
+        }
+    }?;
+
+    let _suffix = scenario.as_str();
 
     // Take MEDIUM scenario here because scenario_fn is to be applied post factum
     let girr_delta_rho_same_curve = get_optional_parameter_array(
         op,
         "girr_delta_rho_same_curve_base",
         &MEDIUM_CORR_SCENARIO.girr_delta_rho_same_curve_base,
-    );
+    )?;
     let girr_delta_rho_diff_curve = get_optional_parameter(
         op,
         "girr_delta_rho_diff_curve_base",
         &MEDIUM_CORR_SCENARIO.girr_delta_rho_diff_curve_base,
-    );
+    )?;
     let girr_delta_rho_infl = get_optional_parameter(
         op,
         "girr_delta_rho_infl_base",
         &MEDIUM_CORR_SCENARIO.girr_delta_rho_infl_base,
-    );
+    )?;
     let girr_delta_rho_xccy = get_optional_parameter(
         op,
         "girr_delta_rho_xccy_base",
         &MEDIUM_CORR_SCENARIO.girr_delta_rho_xccy_base,
-    );
+    )?;
 
     let girr_delta_gamma = get_optional_parameter(
         op,
         format!("girr_delta_gamma{_suffix}").as_str(),
         &scenario.girr_delta_vega_gamma,
-    );
+    )?;
     let girr_delta_gamma_crr2_erm2 = get_optional_parameter(
         op,
         format!("girr_delta_gamma_erm2{_suffix}").as_str(),
         &scenario.girr_delta_vega_gamma_erm2,
-    );
-    let erm2ccys = get_optional_parameter_vec(op, "erm2_ccys", &scenario.erm2_ccys);
+    )?;
+    let erm2ccys = get_optional_parameter_vec(op, "erm2_ccys", &scenario.erm2_ccys)?;
 
-    girr_delta_charge(
+    Ok(girr_delta_charge(
         girr_delta_gamma,
         girr_delta_rho_same_curve,
         girr_delta_rho_diff_curve,
@@ -158,7 +170,7 @@ fn girr_delta_charge_distributor(
         erm2ccys,
         scenario.scenario_fn,
         rep_ccy,
-    )
+    ))
 }
 
 #[allow(clippy::too_many_arguments)]
@@ -432,17 +444,17 @@ pub(crate) fn girr_corr_matrix() -> Array2<f64> {
 /// !Note This is not a real measure, as MAX should be taken as
 /// MAX(ir_delta_low+ir_vega_low+eq_curv_low, ..._medium, ..._high).
 /// This is for convienience view only.
-fn girr_delta_max(op: &OCP) -> Expr {
-    max_exprs(&[
-        girr_delta_charge_low(op),
-        girr_delta_charge_medium(op),
-        girr_delta_charge_high(op),
-    ])
+fn girr_delta_max(op: &CPM) -> PolarsResult<Expr> {
+    Ok(max_exprs(&[
+        girr_delta_charge_low(op)?,
+        girr_delta_charge_medium(op)?,
+        girr_delta_charge_high(op)?,
+    ]))
 }
 /// Exporting Measures
 pub(crate) fn girr_delta_measures() -> Vec<Measure> {
     vec![
-        Measure {
+        Measure::Base(BaseMeasure {
             name: "GIRR DeltaSens".to_string(),
             calculator: Box::new(total_ir_delta_sens),
             aggregation: None,
@@ -451,8 +463,8 @@ pub(crate) fn girr_delta_measures() -> Vec<Measure> {
                     .eq(lit("Delta"))
                     .and(col("RiskClass").eq(lit("GIRR"))),
             ),
-        },
-        Measure {
+        }),
+        Measure::Base(BaseMeasure {
             name: "GIRR DeltaSens Weighted".to_string(),
             calculator: Box::new(girr_delta_sens_weighted),
             aggregation: None,
@@ -461,8 +473,8 @@ pub(crate) fn girr_delta_measures() -> Vec<Measure> {
                     .eq(lit("Delta"))
                     .and(col("RiskClass").eq(lit("GIRR"))),
             ),
-        },
-        Measure {
+        }),
+        Measure::Base(BaseMeasure {
             name: "GIRR DeltaSb".to_string(),
             calculator: Box::new(girr_delta_sb),
             aggregation: Some("scalar"),
@@ -471,8 +483,8 @@ pub(crate) fn girr_delta_measures() -> Vec<Measure> {
                     .eq(lit("Delta"))
                     .and(col("RiskClass").eq(lit("GIRR"))),
             ),
-        },
-        Measure {
+        }),
+        Measure::Base(BaseMeasure {
             name: "GIRR DeltaCharge Low".to_string(),
             calculator: Box::new(girr_delta_charge_low),
             aggregation: Some("scalar"),
@@ -481,8 +493,8 @@ pub(crate) fn girr_delta_measures() -> Vec<Measure> {
                     .eq(lit("Delta"))
                     .and(col("RiskClass").eq(lit("GIRR"))),
             ),
-        },
-        Measure {
+        }),
+        Measure::Base(BaseMeasure {
             name: "GIRR DeltaKb Low".to_string(),
             calculator: Box::new(girr_delta_kb_low),
             aggregation: Some("scalar"),
@@ -491,8 +503,8 @@ pub(crate) fn girr_delta_measures() -> Vec<Measure> {
                     .eq(lit("Delta"))
                     .and(col("RiskClass").eq(lit("GIRR"))),
             ),
-        },
-        Measure {
+        }),
+        Measure::Base(BaseMeasure {
             name: "GIRR DeltaCharge Medium".to_string(),
             calculator: Box::new(girr_delta_charge_medium),
             aggregation: Some("scalar"),
@@ -501,8 +513,8 @@ pub(crate) fn girr_delta_measures() -> Vec<Measure> {
                     .eq(lit("Delta"))
                     .and(col("RiskClass").eq(lit("GIRR"))),
             ),
-        },
-        Measure {
+        }),
+        Measure::Base(BaseMeasure {
             name: "GIRR DeltaKb Medium".to_string(),
             calculator: Box::new(girr_delta_kb_medium),
             aggregation: Some("scalar"),
@@ -511,8 +523,8 @@ pub(crate) fn girr_delta_measures() -> Vec<Measure> {
                     .eq(lit("Delta"))
                     .and(col("RiskClass").eq(lit("GIRR"))),
             ),
-        },
-        Measure {
+        }),
+        Measure::Base(BaseMeasure {
             name: "GIRR DeltaCharge High".to_string(),
             calculator: Box::new(girr_delta_charge_high),
             aggregation: Some("scalar"),
@@ -521,8 +533,8 @@ pub(crate) fn girr_delta_measures() -> Vec<Measure> {
                     .eq(lit("Delta"))
                     .and(col("RiskClass").eq(lit("GIRR"))),
             ),
-        },
-        Measure {
+        }),
+        Measure::Base(BaseMeasure {
             name: "GIRR DeltaKb High".to_string(),
             calculator: Box::new(girr_delta_kb_high),
             aggregation: Some("scalar"),
@@ -531,8 +543,8 @@ pub(crate) fn girr_delta_measures() -> Vec<Measure> {
                     .eq(lit("Delta"))
                     .and(col("RiskClass").eq(lit("GIRR"))),
             ),
-        },
-        Measure {
+        }),
+        Measure::Base(BaseMeasure {
             name: "GIRR DeltaCharge MAX".to_string(),
             calculator: Box::new(girr_delta_max),
             aggregation: Some("scalar"),
@@ -541,6 +553,6 @@ pub(crate) fn girr_delta_measures() -> Vec<Measure> {
                     .eq(lit("Delta"))
                     .and(col("RiskClass").eq(lit("GIRR"))),
             ),
-        },
+        }),
     ]
 }
